import numpy as np
import json
import os

results_dir = '../../Population_Samples/default/'
map_redshift_to_names = {'NoEvolution': '_fixed_redshift/'}
map_vt_to_names = {'FullVT': '_full_vt', 'LowVT': '_low_vt'}
post_fname = '_iid_mag_two_comp_iid_tilt_powerlaw_redshift_result.json'
prior_loss_fname = '_iid_mag_two_comp_iid_tilt_powerlaw_redshift_rejection.txt'

map_mass_model_to_names = {'truncated' : 'mass_b', 'peak' : 'mass_c', 'BPL' : 'mass_d', 'multipeak': 'mass_e', 'BPLPeak': 'mass_f'}

map_BBHsel_to_names = {
    'NoAugNoEvolution': 'default',
    'NoAugNoEvolutionNoPeak': 'no_peak',
    'NoAugNoEvolutionNoSmoothing': 'no_smoothing',
}

def path_to_result_file(BBHsel_name, mass_model):
    mass = map_mass_model_to_names[mass_model]
    BBHsel = map_BBHsel_to_names[BBHsel_name]
    if (BBHsel_name == 'NoAugNoEvolutionNoPeak'):
        return results_dir  + BBHsel + '/o1o2o3_' + mass + post_fname
    elif (BBHsel_name == 'NoAugNoEvolutionNoSmoothing'):
        return results_dir + BBHsel + '/o1o2o3_' + mass + post_fname
    else:
        return results_dir + '/o1o2o3_' + mass + post_fname

def path_to_prior_loss_file(BBHsel_name, mass_model):
    mass = map_mass_model_to_names[mass_model]
    BBHsel = map_BBHsel_to_names[BBHsel_name]
    if (BBHsel_name == 'NoAugEvolutionNoPeak'):
        return  BBHsel + '/o1o2o3_' + mass + prior_loss_fname 
    elif (BBHsel_name == 'NoAugEvolutionNoSmoothing'):
        return  BBHsel + '/o1o2o3_' + mass + prior_loss_fname 
    else:
        return  'o1o2o3_' + mass + prior_loss_fname 


stat_file =  open("bayes_factor_stats.txt","w")

stat_dict = {}
bfs = []
log_bfs = []

for BBHsel_name in ['NoAugNoEvolution', 'NoAugNoEvolutionNoPeak', 'NoAugNoEvolutionNoSmoothing']:
    for mass_model in map_mass_model_to_names.keys():
        if os.path.exists(path_to_result_file(BBHsel_name, mass_model)):
            ff = open(path_to_result_file(BBHsel_name, mass_model))
            data1 = json.load(ff)
            log_bf = data1['log_bayes_factor']
            log_bf_err = data1['log_evidence_err']
            log_bf_max = log_bf + log_bf_err
            log_bf_min = log_bf - log_bf_err

            bf = np.exp(log_bf)
            bf_max = np.exp(log_bf_max)
            bf_min = np.exp(log_bf_min)

            if os.path.exists(path_to_prior_loss_file(BBHsel_name, mass_model)):
                data2 =  np.loadtxt(path_to_prior_loss_file(BBHsel_name, mass_model),delimiter=",")
                correction_factor = (data2[2]-data2[0])/data2[2]

                print("Retrieving bayes factors for "+path_to_result_file(BBHsel_name, mass_model))
                stat_dict[mass_model+BBHsel_name+"BF"] = [
                    "bayes factor for "+path_to_result_file(BBHsel_name, mass_model),
                    correction_factor*bf,
                    correction_factor*bf_max - correction_factor*bf,
                    correction_factor*bf_min - correction_factor*bf,
                ]

                stat_dict[mass_model+BBHsel_name+"LogBF"] = [
                    "log bayes factor for "+path_to_result_file(BBHsel_name, mass_model),
                    np.log10(correction_factor*bf), 
                    np.log10(correction_factor*bf_max) - np.log10(correction_factor*bf),
                    np.log10(correction_factor*bf_max) - np.log10(correction_factor*bf),
                ]

                bfs.append(stat_dict[mass_model+BBHsel_name+"BF"][1])
                log_bfs.append(stat_dict[mass_model+BBHsel_name+"LogBF"][1])
                
            else:
                print(path_to_prior_loss_file(BBHsel_name, mass_model), ' does not exist')
        else:
            print(path_to_result_file(BBHsel_name, mass_model), ' does not exist')

max_bf = max(bfs)
max_log_bf = max(log_bfs)

for key in stat_dict:
    stat_file.write("% "+stat_dict[key][0]+"\n")
    if "LogBF" in key:
        stat_file.write("\\newcommand{\\%s}{\ensuremath{%s}}\n" % (key, round(stat_dict[key][1]-max_log_bf,2)))
    else:
        stat_file.write("\\newcommand{\\%s}{\ensuremath{%s}}\n" % (key, round(stat_dict[key][1]/max_bf,2)))
    stat_file.write(" \n")
        
stat_file.close()
